// CAdd.cp -- dialog methods

#include "CAdd.h"

#include <UEnvironment.h>
#include <UReanimator.h>
#include <URegistrar.h>
#include <LStream.h>
#include <LTabGroup.h>
#include <LPushButton.h>
#include <LAMPushButtonImp.h>
#include <LGAPushButtonImp.h>
#include <LPictureControl.h>
#include <LAMControlImp.h>
#include <LGAPictureControlImp.h>
#include <LStaticText.h>
#include <LAMStaticTextImp.h>
#include <LGAStaticTextImp.h>
#include <LClock.h>
#include <LEditText.h>
#include <LAMEditTextImp.h>
#include <LGAEditTextImp.h>
#include <LCheckBox.h>
#include <LGACheckboxImp.h>
#include <LPopupButton.h>
#include <LAMPopupButtonImp.h>
#include <LGAPopupButtonImp.h>
#include <CTextUtils.h>

#include "DReminder.h"
#include "AMReminderCmds.h"
#include <PP_Messages.h>

const MessageT	msgDate2	= 'Date';
const MessageT	msgTime2	= 'Time';
const MessageT	msgMessage2	= 'Msg ';
const MessageT	msgDisplayIcon	= 'Icon';
const MessageT	msgDisplayAlert	= 'Alrt';
const MessageT	msgPlaySound	= 'Play';
const MessageT	msgSoundPopup	= 'Soup';

#define PPob_AddID	202
#define RidL_AddID	202

Boolean		CAdd::sIsRegistered = false;

//----------
CAdd*		CAdd::CreateAdd (
	LCommander*		inSuperCommander,
	CommandT		inCommand,
	DReminder*		inData)
{
	if (!sIsRegistered) {
		RegisterClass ();
	}
	CAdd*		dlog;
	dlog = ((CAdd *)LWindow::CreateWindow(PPob_AddID, inSuperCommander));
	dlog->mCommand = inCommand;
	dlog->SetFromData (inData);

	return dlog;
}

//----------
#define	RegisterClasses(AbstractClass,AMImpClass,GAImpClass)	\
	RegisterClass_(AbstractClass);	\
	if (useAppearance) {	\
		RegisterClassID_(AMImpClass, AbstractClass::imp_class_ID);	\
	} else {	\
		RegisterClassID_(GAImpClass, AbstractClass::imp_class_ID);	\
	}

//----------
void	CAdd::RegisterClass ()
{
	Boolean		useAppearance = UEnvironment::HasFeature (env_HasAppearance);

	RegisterClass_(CAdd);

	// register the pane classes we use
	RegisterClasses (LPushButton, LAMPushButtonImp, LGAPushButtonImp);
	RegisterClasses (LPictureControl, LAMControlImp, LGAPictureControlImp);
	RegisterClasses (LStaticText, LAMStaticTextImp, LGAStaticTextImp);
	RegisterClasses (LClock, LAMControlImp, LAMControlImp);
	RegisterClasses (LEditText, LAMEditTextImp, LGAEditTextImp);
	RegisterClasses (LCheckBox, LAMControlImp, LGACheckBoxImp);
	RegisterClasses (LPopupButton, LAMPopupButtonImp, LGAPopupButtonImp);

	sIsRegistered = true;
}

//----------
CAdd::CAdd (
	LStream*	inStream)
	: LGADialog (inStream)
{
	mData = nil;
}

//----------
CAdd::~CAdd()
{
	delete mData;
}

//----------
//	This member function gets called once the containment hierarchy that contains
//	this pane has been built. It gives us a chance to get data members for
//	interesting subviews, and to do other operations now that our subviews exist.
void	CAdd::FinishCreateSelf()
{
	LGADialog::FinishCreateSelf();

	mOKButton = (LPushButton*) FindPaneByID ('OK  ');

	mCancelButton = (LPushButton*) FindPaneByID ('Canl');

	mDate2Field = (LClock*) FindPaneByID ('Date');
	mDate2Field->AddListener (this);

	mTime2Field = (LClock*) FindPaneByID ('Time');
	mTime2Field->AddListener (this);

	mMessage2Field = (LEditText*) FindPaneByID ('Msg ');
	mMessage2Field->AddListener (this);

	mDisplayIconCheck = (LCheckBox*) FindPaneByID ('Icon');

	mDisplayAlertCheck = (LCheckBox*) FindPaneByID ('Alrt');

	mPlaySoundCheck = (LCheckBox*) FindPaneByID ('Play');

	mSoundPopupPopup = (LPopupButton*) FindPaneByID ('Soup');


	LTabGroup*		tabGroup = new LTabGroup(this);
	mDate2Field->SetSuperCommander(tabGroup);	// becomes the active field
	mTime2Field->SetSuperCommander(tabGroup);
	mMessage2Field->SetSuperCommander(tabGroup);

	UReanimator::LinkListenerToControls(this, this, RidL_AddID);
		// the purpose is to "connect" self to whatever controls
		// that we want to "listen" to

// any additional initialization for your dialog:

}

//----------
void	CAdd::SetFromData (
	DReminder*		inData)
{
	mData = inData;
	mData->AddListener (this);

	{
		LongDateRec		dateTime = mData->GetYearMonthDay ();
		mDate2Field->SetLongDate (dateTime);
	}
	{
		LongDateRec		dateTime = mData->GetHourMinute ();
		mTime2Field->SetLongDate (dateTime);
	}
	CTextUtils::SetText (mMessage2Field, mData->GetMessage ());
	mDisplayIconCheck->SetValue (mData->GetShowIcon ());
	mDisplayAlertCheck->SetValue (mData->GetShowAlert ());
	mPlaySoundCheck->SetValue (mData->GetPlaySound ());
	mSoundPopupPopup->SetValue (mData->GetSoundIndex ());
}

//----------
DReminder*		CAdd::GetData ()
{

	return mData;
}

//----------
void	CAdd::DataChanged (
	long		inDataID)
{
	StopListening ();

	if (inDataID == idYearMonthDay) {
		LongDateRec		dateTime = mData->GetYearMonthDay ();
		mDate2Field->SetLongDate (dateTime);
	}
	if (inDataID == idHourMinute) {
		LongDateRec		dateTime = mData->GetHourMinute ();
		mTime2Field->SetLongDate (dateTime);
	}
	if (inDataID == idMessage) {
		if (!mMessage2Field->IsTarget ()) {
			CTextUtils::SetText (mMessage2Field, mData->GetMessage ());
		}
	}
	if (inDataID == idShowIcon) {
		mDisplayIconCheck->SetValue (mData->GetShowIcon ());
	}
	if (inDataID == idShowAlert) {
		mDisplayAlertCheck->SetValue (mData->GetShowAlert ());
	}
	if (inDataID == idPlaySound) {
		mPlaySoundCheck->SetValue (mData->GetPlaySound ());
	}
	if (inDataID == idSoundIndex) {
		mSoundPopupPopup->SetValue (mData->GetSoundIndex ());
	}

	StartListening ();
}

//----------
void	CAdd::ListenToMessage (
	MessageT	inMessage,
	void		*ioParam)
{
	switch (inMessage) {
	case msg_OK:
			GetSuperCommander()->ProcessCommand(-mCommand, this);
		break;

	case msg_Cancel:
			DoClose();
		break;

	case msgDataChanged:
			DataChanged ((long)ioParam);
		break;

	case msgDate2:
		{
			LongDateRec		dateTime;

			mDate2Field->GetLongDate (dateTime);
			mData->SetYearMonthDay (dateTime);
		}
		break;

	case msgTime2:
		{
			LongDateRec		dateTime;

			mTime2Field->GetLongDate (dateTime);
			mData->SetHourMinute (dateTime);
		}
		break;

	case msgMessage2:
		{
			Str255		string;

			mMessage2Field->GetDescriptor (string);
			mData->SetMessage (string);
		}
		break;

	case msgDisplayIcon:
			mData->SetShowIcon (mDisplayIconCheck->GetValue ());
		break;

	case msgDisplayAlert:
			mData->SetShowAlert (mDisplayAlertCheck->GetValue ());
		break;

	case msgPlaySound:
			mData->SetPlaySound (mPlaySoundCheck->GetValue ());
		break;

	case msgSoundPopup:
			mData->SetSoundIndex (mSoundPopupPopup->GetValue ());
		break;


	default:
		  ; // do something
		break;
	}
}

//----------
Boolean		CAdd::ObeyCommand (
	CommandT	inCommand,
	void*		ioParam)
{
	Boolean		cmdHandled = true;

	if (inCommand < 0) {
		// modal dialog has finished

		switch (-inCommand) {
		}

	} else {
		switch (inCommand) {

		default:
				cmdHandled = LGADialog::ObeyCommand(inCommand, ioParam);
			break;
		}
	}

	return cmdHandled;
}

//----------
void	CAdd::FindCommandStatus (
	CommandT	inCommand,
	Boolean		&outEnabled,
	Boolean		&outUsesMark,
	Char16		&outMark,
	Str255		outName)
{
	outUsesMark = false;

	switch (inCommand) {

	// +++ Add cases here for the commands you handle

	default:
			LGADialog::FindCommandStatus(inCommand, outEnabled,
											outUsesMark, outMark, outName);
		break;
	}
}

// following functions will be obsoleted
// retained only for backwards compatibility
	/*
	//----------
	Date
	CAdd::GetDate2FieldDate ()
	{
	}

	//----------
	void
	CAdd::SetDate2FieldDate (
		Date		str)
	{
	}
	*/

	/*
	//----------
	Date
	CAdd::GetTime2FieldDate ()
	{
	}

	//----------
	void
	CAdd::SetTime2FieldDate (
		Date		str)
	{
	}
	*/

//----------
void
CAdd::GetMessage2FieldString (
	Str255		str)
{
	mMessage2Field->GetDescriptor (str);
}

//----------
void
CAdd::SetMessage2FieldString (
	ConstStr255Param		str)
{
	mMessage2Field->SetDescriptor (str);
}

//----------
Boolean
CAdd::GetDisplayIconChoice()
{
	return mDisplayIconCheck->GetValue();
}

void
CAdd::SetDisplayIconChoice (
	Boolean		inChoice)
{
	mDisplayIconCheck->SetValue (inChoice);
}

//----------
Boolean
CAdd::GetDisplayAlertChoice()
{
	return mDisplayAlertCheck->GetValue();
}

void
CAdd::SetDisplayAlertChoice (
	Boolean		inChoice)
{
	mDisplayAlertCheck->SetValue (inChoice);
}

//----------
Boolean
CAdd::GetPlaySoundChoice()
{
	return mPlaySoundCheck->GetValue();
}

void
CAdd::SetPlaySoundChoice (
	Boolean		inChoice)
{
	mPlaySoundCheck->SetValue (inChoice);
}

//----------
short
CAdd::GetSoundPopupPopupChoice ()
{
	return mSoundPopupPopup->GetValue();
}

void
CAdd::SetSoundPopupPopupChoice (
	short		choice)
{
	mSoundPopupPopup->SetValue (choice);
}

