// AMListBox.cp

// usage:
//	Create a list using our custom LDEF.
//	The custom LDEF calls a class static function
//	which in turn calls instance functions.
//
//	ListHandle		list = ...
//	AMListBox		listObj;	// on stack for automatic
//								// construction/destruction
//
//	listObj.Initialize (list);

#include <Types.h>
#include <Controls.h>
#include <Lists.h>
#include <TextEdit.h>

#include "AMArray.h"
#include "AMPanel.h"

#include "AMListBox.h"

//----------
AMListBox::AMListBox ()
{
	mListHandle = nil;
	mUserData.defProc = nil;
	mUserData.listBox = this;
	mArray = nil;
	mPanel = nil;
	mSaveClip = NewRgn ();
	mTempRgn = NewRgn ();
}

//----------
AMListBox::~AMListBox ()
{
	UniversalProcPtr	upp;

	upp = (UniversalProcPtr)mUserData.defProc;
	if (upp != nil) {
		DisposeRoutineDescriptor (upp);
		mUserData.defProc = nil;
	}
	if (mListHandle != nil) {
		// normally should be nil because Close'd
		(**mListHandle).userHandle = nil;
		// so LDEF won't try to call us
	}
	if (mArray != nil) {
		mArray->RemoveUser (this);
	}
	if (mSaveClip != nil) {
		DisposeRgn (mSaveClip);
	}
	if (mTempRgn != nil) {
		DisposeRgn (mTempRgn);
	}
}

//----------
void	AMListBox::Initialize (
	ListHandle		inListHandle)
{
	mListHandle = inListHandle;
	mUserData.defProc = NewListDefProc (AMListBox::StaticDefProc);
	if (mListHandle != nil) {
		// could assert not nil
		(**mListHandle).userHandle = (Handle)(&mUserData);
	}
}

//----------
void	AMListBox::UseArray (
	AMArray*		inArray)
{
	if (mArray != nil) {
		mArray->RemoveUser (this);
	}

	mArray = inArray;

	if (mArray != nil) {
		mArray->AddUser (this);

		if (mListHandle != nil) {
			short		numRows = mArray->GetCount ();

			LDelRow (32767, 0, mListHandle);
			LAddRow (numRows, 0, mListHandle);

			// fix bug in listbox / scrollbar
			// listbox erroneously activates the scrollbar
			// when max < min
			// which happens if numRows < height available
			ControlHandle	vScroll = (**mListHandle).vScroll;
			if (vScroll != nil) {
				SInt16		min = GetControlMinimum (vScroll);
				SInt16		max = GetControlMaximum (vScroll);
				if (max < min) {
					SetControlMaximum (vScroll, min);
				}
			}
		}	// if mListHandle
	}	// if mArray
}

//----------
void	AMListBox::UsePanel (
	AMPanel*		inPanel)
{
	mPanel = inPanel;

	if (mSaveClip == nil) {
		mSaveClip = NewRgn ();
	}
	if (mTempRgn == nil) {
		mTempRgn = NewRgn ();
	}
}

//----------
void	AMListBox::Draw (
	Boolean		inSelect,
	Rect*		inRect,
	Cell		inCell,
	short		inDataOffset,
	short		inDataLen)
{
	DrawRow (inRect, inCell.v);
	if (inSelect) {
		HiliteRect (inRect);
	}
}

//----------
void	AMListBox::DrawRow (
	Rect*		inRect,
	short		inRow)	// 0-based
{
	void*		element = nil;

	if (mArray != nil) {
		element = mArray->GetNth (inRow);
	}
	if (mPanel != nil) {
		ControlHandle	panel = mPanel->GetPanelRoot ();

//		::HideControl (panel);
		::SetControlVisibility (panel, false, false);
		mPanel->MoveTo (*inRect);
		mPanel->ConnectToData ((AMSignaler*)element);
	//	::ShowControl (panel);
		::SetControlVisibility (panel, true, false);

		::GetClip (mSaveClip);
		::RectRgn (mTempRgn, inRect);
		::SectRgn (mSaveClip, mTempRgn, mTempRgn);
		::SetClip (mTempRgn);
		mPanel->Draw ();
		::SetClip (mSaveClip);
//		::HideControl (panel);
		::SetControlVisibility (panel, false, false);
	}
}

//----------
void	AMListBox::Hilite (
	Boolean		inSelect,
	Rect*		inRect,
	Cell		inCell,
	short		inDataOffset,
	short		inDataLen)
{
	HiliteRect (inRect);
}

//----------
void	AMListBox::HiliteRect (
	Rect*		inRect)
{
	LMSetHiliteMode (LMGetHiliteMode () & (~(1 << hiliteBit)));
	::InvertRect (inRect);   // highlight cell rectangle
}

//----------
void	AMListBox::Close ()
{
	mListHandle = nil;
}

//----------
void	AMListBox::ArrayChanged (
	AMArray*		inWho,
	EChangedHow		inHow,
	UInt32			inWhere)
{
	if ((inWho == mArray)	// who else?
	&&  (mListHandle != nil)) {

		LSetDrawingMode (true, mListHandle);

		switch (inHow) {
		case itemInserted:
				LAddRow (1, inWhere, mListHandle);
			break;
		case itemRemoved:
				LDelRow (1, inWhere, mListHandle);
			break;
		case itemChanged:
				Cell	cell;

				cell.v = inWhere;
				cell.h = 0;
				LSetCell (nil, 0, cell, mListHandle);
			break;
		}

		// fix bug in listbox / scrollbar
		// listbox erroneously activates the scrollbar
		// when max < min
		// which happens if numRows < height available
		ControlHandle	vScroll = (**mListHandle).vScroll;
		if (vScroll != nil) {
			SInt16		min = GetControlMinimum (vScroll);
			SInt16		max = GetControlMaximum (vScroll);
			if (max < min) {
				SetControlMaximum (vScroll, min);
			}
		}
	}
}

//----------
// static
pascal void		AMListBox::StaticDefProc (
	short		inMessage,
	Boolean		inSelect,
	Rect*		inRect,
	Cell		inCell,
	short		inDataOffset,
	short		inDataLen,
	ListHandle	inHandle)
{
	UserDataPtr		userData;
	AMListBox*		listBox;

	userData = (UserDataPtr)((**inHandle).userHandle);
	listBox = userData->listBox;

	switch (inMessage) {
	case lDrawMsg:
			listBox->Draw (inSelect, inRect, inCell, inDataOffset, inDataLen);
		break;
	case lHiliteMsg:
			listBox->Hilite (inSelect, inRect, inCell, inDataOffset, inDataLen);
		break;
	case lCloseMsg:
			listBox->Close ();
		break;
	default:
			userData = userData;
		break;
	}; // switch
}
