/*  1998, Bowers Development Corp. */
/* ControlUtils.cp */

#include <Types.h>
#include <Quickdraw.h>
#include <Controls.h>
#include <Dialogs.h>
#include <Events.h>
#include <Fonts.h>
#include <fp.h>
#include <Icons.h>
#include <Lists.h>
#include <Menus.h>
#include <Resources.h>
#include <TextEdit.h>
#include <ToolUtils.h>
#include <Appearance.h>

#include "Globals.h"

#include "ControlUtils.h"

//----------
struct wftbItemRec {
	SInt16 					key;	// 1 => dataItem
	ControlFontStyleRec		fontRec;
	Str255					fontName;

public:
	wftbItemRec*	nextItem () const;
};
typedef struct wftbItemRec		wftbItemRec;
typedef wftbItemRec*	wftbItemPtr;

//----------
inline	wftbItemRec*	wftbItemRec::nextItem () const
{
	if (key == 1) {
		return (wftbItemRec*)&fontName [1 + fontName [0]];
	} else {
		return (wftbItemRec*)&fontRec;
	}
};		

//----------
struct wftbRec {
	SInt16 			version;	// must be 0
	SInt16 			numItems;
	wftbItemRec		firstItem;
};
typedef struct wftbRec		wftbRec;
typedef wftbRec*	wftbPtr;
typedef wftbPtr*	wftbHandle;

//----------
void	SetWindowItemFont (
	ControlHandle	inControl,
	Handle			inWftb,
	SInt16			inItemNum)
{
	if (inWftb != nil) {
		wftbPtr			wftb = *(wftbHandle)inWftb;

		if ((wftb->version == 0)
		&&  (inItemNum <= wftb->numItems)) {
			wftbItemPtr		itemPtr;
			SInt16			i;
			ControlFontStyleRec		fontRec;

			itemPtr = &wftb->firstItem;
			for (i = 1; i < inItemNum; i++) {
				itemPtr = itemPtr->nextItem ();
			} // for

			if (itemPtr->key == 1) {
				fontRec = itemPtr->fontRec;
				if ((fontRec.flags & kDialogFontUseFontNameMask) != 0) {
					::GetFNum (itemPtr->fontName, &fontRec.font);
					fontRec.flags -= kDialogFontUseFontNameMask;
				}
				::SetControlFontStyle (inControl, &fontRec);
			}
		}
	}
}

//----------
inline	SInt32	Min (
	SInt32		inA,
	SInt32		inB)
{
	if (inA < inB) {
		return inA;
	} else {
		return inB;
	}
}

//----------
CharsHandle		GetEditTextChars (
	ControlHandle	inControl)
{
	CharsHandle		result = nil;
	TEHandle		te;
	Size			actualSize = 0;

	if (GetControlData (inControl,
					 kControlNoPart,
					 kControlEditTextTEHandleTag,
					 sizeof (TEHandle),
					 (Ptr)&te,
					 &actualSize) == 0) {
		result = TEGetText (te);
	}
	return result;
}

//----------
void	GetEditTextPasswordText (
	ControlHandle	inControl,
	Str255			outStr)
{
	Size		actualSize = 0;
	
	GetControlData	(inControl,
					 kControlNoPart,
					 kControlEditTextPasswordTag,
					 255,
					 (Ptr)&outStr [1],
					 &actualSize);
	outStr [0] = (unsigned char)(Min (255, actualSize));
}

//----------
void	GetControlText (
	ControlHandle	inControl,
	Str255			outStr)
{
	Size		actualSize = 0;

	GetControlData	(inControl,
					 kControlNoPart,
					 kControlEditTextTextTag,
					 255,
					 (Ptr)&outStr [1],
					 &actualSize);
	outStr [0] = (unsigned char)(Min (255, actualSize));
}

//----------
void	SetControlText (
	ControlHandle		inControl,
	ConstStr255Param	inStr)
{
	SetControlData	(inControl,
					 kControlNoPart,
					 kControlStaticTextTextTag,
					 StrLength (inStr),
					 (Ptr)&inStr [1]);
	DrawControlInCurrentPort (inControl);
}

//----------
void	SetControlText (
	ControlHandle		inControl,
	const ansiString&	inStr)
{
	SetControlData	(inControl,
					 kControlNoPart,
					 kControlStaticTextTextTag,
					 (long)inStr.length (),
					 (Ptr)inStr.c_str());
	DrawControlInCurrentPort (inControl);
}

//----------
void	SetControlFromTEXT (
	ControlHandle		inControl,
	short				inTEXTid)
{
	Handle			text;

	text = ::GetResource ('TEXT', inTEXTid);
	if (text != nil) {
		LoadResource (text);
		HLock (text);

		SetControlData	(inControl,
						 kControlNoPart,
						 kControlStaticTextTextTag,
						 ::GetHandleSize (text),
						 &(**text));
		DrawControlInCurrentPort (inControl);

		HUnlock (text);
	}
}

//----------
SInt32		GetControlTextValue (
	ControlHandle	inControl)
{
	SInt32		result;
	Str255		str;

	result = 0;
	GetControlText (inControl, str);

	if (StrLength (str) > 0) {
		StringToNum (str, &result);
	}
	return result;
}

//----------
void	SetControlTextValue (
	ControlHandle	inControl,
	SInt32			inValue)
{
	Str255		str;

	NumToString (inValue, str);
	SetControlText (inControl, str);
}

//----------
double_t	GetControlTextFloat (
	ControlHandle	inControl)
{
	double_t	result;
	Str255		str;
	short		index = 0;
	decimal		decRecord;
	short		vp;

	result = 0;
	GetControlText (inControl, str);

	::str2dec (p2cstr (str), &index, &decRecord, &vp);

	if (vp > 0) {
		result = ::dec2num (&decRecord);
	}

	return result;
}

//----------
void	SetControlTextFloat (
	ControlHandle	inControl,
	double_t		inValue,
	char			inStyle,	/*  FLOATDECIMAL or FIXEDDECIMAL */
	short			inDigits)
{
	decform		format;
	format.style = inStyle;
	format.digits = inDigits;

	decimal		decRecord;
	::num2dec (&format, inValue, &decRecord);
	
	char	numStr [DECSTROUTLEN + 1];
	::dec2str (&format, &decRecord, numStr);
	
	SetControlText (inControl, c2pstr (numStr));
}

//----------
void	SetLayerGroupValue (
	ControlHandle	inControl,
	SInt16			inValue)	// 1-based
{
	UInt16			numLayers = 0;
	UInt16			i;
	ControlHandle	layer;

	CountSubControls (inControl, &numLayers);
	for (i = 1; i <= numLayers; i++) {
		GetIndexedSubControl (inControl, i, &layer);
		if (i == inValue) {
			ActivateControl (layer);
			ShowControl (layer);
		} else {
			HideControl (layer);
			DeactivateControl (layer);
		}
	} // for
	DrawControlInCurrentPort (inControl);
}

//----------
void	SetDefaultState (
	ControlHandle	inControl,
	Boolean			inIsDefault)
{
	SetControlData	(inControl,
					 kControlNoPart,
					 kControlPushButtonDefaultTag,
					 sizeof (inIsDefault),
					 (Ptr)&inIsDefault);
}

//----------
void	SetIndeterminateState (
	ControlHandle	inControl,
	Boolean			inIsIndeterminate )
{
	SetControlData	(inControl,
					 kControlNoPart,
					 kControlProgressBarIndeterminateTag,
					 sizeof (inIsIndeterminate),
					 (Ptr)&inIsIndeterminate);
}

//----------
LongDateRec		GetClockDateTime (
	ControlHandle	inControl)
{
	LongDateRec		result;
	Size			actualSize;
		
	GetControlData	(inControl,
					 kControlClockPart,
					 kControlClockLongDateTag,
					 sizeof (LongDateRec),
					 (Ptr)&result,
					 &actualSize);
	return result;
}

//----------
void	SetClockDateTime (
	ControlHandle	inControl,
	LongDateRec		inTime)
{
	SetControlData	(inControl,
					 kControlClockPart,
					 kControlClockLongDateTag,
					 sizeof (inTime),
					 (Ptr)&inTime);
}

//----------
ListHandle		GetListHandle (
	ControlHandle	inControl)
{
	ListHandle		result = nil;
	Size			actualSize = 0;

	GetControlData	(inControl,
					 kControlNoPart,
					 kControlListBoxListHandleTag,
					 sizeof (result),
					 (Ptr)&result,
					 &actualSize);
	return result;
}

//----------
SInt16		GetListBoxChoice (
	ControlHandle	inControl)
{
	SInt16			result = -1;
	ListHandle		list;
	Cell			selectedCell;

	list = GetListHandle (inControl);

	if (list != nil) {
		SetPt (&selectedCell, 0, 0);
		if (LGetSelect (true, &selectedCell, list)) {
			result = selectedCell.v;
		}
	}

	return result;
}

//----------
void	SetListBoxChoice (
	ControlHandle	inControl,
	SInt16			inValue)	// 0-based
{
	ListHandle		list;
	Cell			selectedCell;

	list = GetListHandle (inControl);

	if (list != nil) {
		SetPt (&selectedCell, 0, inValue);
		LSetSelect (true, selectedCell, list);
	}
}

//----------
void	GetListRow (
	Str255			data,
	SInt16			index,
	ListHandle		list)
{
	Cell			selectedCell;
	SInt16			dataLen;

	SetPt (&selectedCell, 0, index);
	dataLen = 255;		// var parameter to LGetCell
	LGetCell (&data [1], &dataLen, selectedCell, list);
	data [0] = (unsigned char) dataLen;
}

//----------
void	SetListRow (
	Str255			data,
	SInt16			index,
	ListHandle		list)
{
	Cell			selectedCell;

	SetPt (&selectedCell, 0, index);
	LSetCell (&data [1], data [0], selectedCell, list);
}

//----------
void	AddToList (
	Str255			data,
	ListHandle		list)
{
	SInt16			newRow;
	
	const SInt16	maxint = 32767;

	newRow = LAddRow (1, maxint, list);
	SetListRow (data, newRow, list);
}

//----------
Boolean		WasDoubleClick (
	ControlHandle	inControl)
{
	Boolean		result = false;
	Size		actualSize = 0;

	GetControlData	(inControl,
					 kControlNoPart,
					 kControlListBoxDoubleClickTag,
					 sizeof (result),
					 (Ptr)&result,
					 &actualSize);
	return result;
}

//----------
Boolean		TrackClick (
	ControlHandle	inControl,
	Point			inWhere)
{
	return (HandleControlClick (inControl, inWhere,
								(short)curEvent.modifiers, nil) != 0);
}

//----------
Boolean		TrackCheckbox (
	ControlHandle	inControl,
	Point			inWhere)
{
	if (TrackClick (inControl, inWhere) != 0) {
		ToggleCheckbox (inControl);
		return true;
	} else {
		return false;
	}
}

//----------
Boolean		ToggleCheckbox (
	ControlHandle	inControl)
{
	SInt16			value;

	value = GetControlValue (inControl);
	value = (value != 1);
			// if value == off or mixed then on
			// if value == on then off
	SetControlValue (inControl, value);

	return (Boolean)value;
}

//----------
Boolean		TrackMulti (
	ControlHandle	inControl,
	Point			inWhere)
{
	if (TrackClick (inControl, inWhere) != 0) {
		BumpMultiState (inControl);
		return true;
	} else {
		return false;
	}
}

//----------
SInt16		BumpMultiState (
	ControlHandle	inControl)
{
	SInt16			value;

	value = GetControlValue (inControl);
	if (value == GetControlMaximum (inControl)) {
		value = GetControlMinimum (inControl);
	} else {
		value++;
	}
	SetControlValue (inControl, value);
	return value;
}

//----------
void	SimulateClick (
	ControlHandle	inControl)
{
	unsigned long		finalTicks;

	HiliteControl (inControl, kControlButtonPart);
	Delay (8, &finalTicks);
	HiliteControl (inControl, 0);
}

//----------
void	EnableControl (
	ControlHandle	inControl,
	Boolean			inEnabled)
{
	if (inEnabled) {
		ActivateControl (inControl);
	} else {
		DeactivateControl (inControl);
	}
}

