// AMPanelTableView.cp

// usage:

#include "AMPanelTableView.h"

#include <UTableHelpers.h>
#include <LStream.h>
#include <LTableArrayStorage.h>
#include "AMArray.h"
#include "AMDataDef.h"
#include "AMPanel.h"

//----------
AMPanelTableView::AMPanelTableView ()
{
	Initialize ();
}

//----------
AMPanelTableView::AMPanelTableView (
	const SPaneInfo&	inPaneInfo,
	const SViewInfo&	inViewInfo)
	: LTableView (inPaneInfo, inViewInfo)
{
	Initialize ();
}

//----------
AMPanelTableView::AMPanelTableView (
	LStream*			inStream)
	: LTableView (inStream)
{
	Initialize ();
}

//----------
void	AMPanelTableView::Initialize ()
{
	mSingleClickMessage = msg_Nothing;
	mDoubleClickMessage = msg_Nothing;
	mPanel = nil;
	mClipRegion = ::NewRgn ();
}

//----------
AMPanelTableView::~AMPanelTableView ()
{
	if (mClipRegion != nil) {
		::DisposeRgn (mClipRegion);
	}
}

//----------
void	AMPanelTableView::UsePanel (
	AMPanel*	inPanel)
{
	mPanel = inPanel;
}

//----------
void	AMPanelTableView::UseArray (
	AMArray*	inArray)
{
	delete mTableStorage;	// in case already have storage
	mTableStorage = nil;

	if (inArray != nil) {
		SetTableStorage (new LTableArrayStorage (this, inArray));
		inArray->AddListener (this);
	}
}
// inArray should be a TArray<someClass*>

//----------
void	AMPanelTableView::ListenToMessage (
	MessageT	inMessage,
	void*		ioParam)
{
	switch (inMessage) {
		case msgArrayChanged:
			SArrayChangedMessage*	message;
			TableIndexT		rowNum;
			LTableStorage*	storage = mTableStorage;
			mTableStorage = nil;

			message = (SArrayChangedMessage*)ioParam;
			rowNum = message->where;
				// both indexes are 1-based

			switch (message->how) {
			case itemInserted:
					if (mCols == 0) {
						InsertCols (1, 0);
					}
					InsertRows (1, rowNum - 1, nil, 0, true);
				break;
			case itemRemoved:
					RemoveRows (1, rowNum, true);
				break;
			case itemChanged:
					STableCell		cell;
					cell.SetCell (rowNum, 1);
					RefreshCell (cell);
				break;
			}
			mTableStorage = storage;
		break;
	}
}

//----------
void	AMPanelTableView::DrawCell (
	const STableCell&	inCell,
	const Rect&			inLocalRect)
{
	AMDataDef*	element = nil;
	UInt32		dataSize = sizeof (AMDataDef);

	GetCellData (inCell, &element, dataSize);
	if (dataSize != sizeof (element)) {
		element = nil;
	}
	if (mPanel != nil) {
		mPanel->PlaceInSuperImageAt (inLocalRect.left,
									 inLocalRect.top,
									 false);
		mPanel->ResizeFrameTo (inLocalRect.right - inLocalRect.left,
								inLocalRect.bottom - inLocalRect.top,
								false);

		mPanel->ConnectToData (element);
		mPanel->Show ();

		Rect	portRect = inLocalRect;
		LocalToPortPoint (topLeft(portRect));
		LocalToPortPoint (botRight(portRect));
		::RectRgn (mClipRegion, &portRect);

		mPanel->Draw (mClipRegion);
		mPanel->Hide ();
		mPanel->DontRefresh (true);
	}
}

//----------
void	AMPanelTableView::ClickCell (
	const STableCell&		inCell,
	const SMouseDownEvent&	inMouseDown)
{
	if (GetClickCount () == 1) {
		SingleClicked ();
	} else {
		DoubleClicked ();
	}
}

//----------
void	AMPanelTableView::SingleClicked ()
{
	if (mSingleClickMessage != msg_Nothing) {
		BroadcastMessage (mSingleClickMessage, this);
	}
}

//----------
void	AMPanelTableView::DoubleClicked ()
{
	if (mDoubleClickMessage != msg_Nothing) {
		BroadcastMessage (mDoubleClickMessage, this);
	}
}

//----------
MessageT	AMPanelTableView::GetDoubleClickMessage () const
{
	return mDoubleClickMessage;
}

//----------
void	AMPanelTableView::SetDoubleClickMessage (
	MessageT	inMessage)
{
	mDoubleClickMessage = inMessage;
}

//----------
MessageT	AMPanelTableView::GetSingleClickMessage () const
{
	return mSingleClickMessage;
}

//----------
void	AMPanelTableView::SetSingleClickMessage (
	MessageT	inMessage)
{
	mSingleClickMessage = inMessage;
}

//----------
SInt32		AMPanelTableView::GetValue() const
{
	return (SInt32) mTableSelector->GetFirstSelectedRow () - 1;  // Like LListBox, 0-based
}

//----------
void	AMPanelTableView::SetValue(
	SInt32		inValue)  // 0-based
{
	mTableSelector->UnselectAllCells ();
	mTableSelector->SelectCell (STableCell ((TableIndexT) inValue+1, 1));  // first cell in 1-based row
	// NOTE if you want whole rows selected, use a Row selector, eg: LTableSingleRowSelector
	// or LTableEditableSelector
}

//----------
void	AMPanelTableView::UnselectAllCells()
{
	LTableView::UnselectAllCells();
	SingleClicked ();	// because value changed
}
